@@ -5,13 +5,12 @@
 import os, sys, requests, time
 
 # Soothsayer Utils
+from soothsayer.utils import add_objects_to_globals
 import soothsayer_utils as syu
 functions_from_soothsayer_utils = [
     "read_url",
 ]
-
-for function_name in functions_from_soothsayer_utils:
-    globals()[function_name] = getattr(syu, function_name)
+add_objects_to_globals(syu, functions_from_soothsayer_utils, globals(), add_version=True, __all__=None)
 
 
 # ======
